﻿using Jst.Lims.Entities;
using Jst.Lims.LaxiaSdk.Entities;
using Kivii;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Jst.Lims.LaxiaSdk.Extensions
{
    public class Detail
    {
        [ApiMember(Description = "测试规格")]
        public string Name { get; set; }

        [ApiMember(Description = "编码")]
        public string Code { get; set; }
        [ApiMember(Description = "部位")]
        public string Part { get; set; }
    }
    public class ConfirmResponse
    {
        public int ResponseCode { get; set; }
        public string ResponseMsg { get; set; }
    }

    //拉夏发送数据接口
    [Route("/LaXia/Api")]
    public class LaxiaOrderConfirm : RestfulExecution<LaXiaOrder>
    {
        public string Token { get; set; }

        [ApiMember(Description = "检测单号")]
        public string CheckNumbers { get; set; }

        [ApiMember(Description = "质量检测")]
        public string Name { get; set; }

        [ApiMember(Description = "委托品牌")]
        public string CBrand { get; set; }

        [ApiMember(Description = "品牌")]
        public string Brand { get; set; }

        [ApiMember(Description = "款号")]
        public string Style { get; set; }

        [ApiMember(Description = "执行标准")]
        public string Standard { get; set; }

        [ApiMember(Description = "检测级别")]
        public string TestLevel { get; set; }

        [ApiMember(Description = "面料品质样")]
        public string Quarity { get; set; }

        [ApiMember(Description = "提供面料成分")]
        public string Contains { get; set; }

        [ApiMember(Description = "付款方式")]
        public string PayType { get; set; }

        [ApiMember(Description = "测试要求")]
        public string TestType { get; set; }

        [ApiMember(Description = "检测所编号")]
        public string TestInstitute { get; set; }

        [ApiMember(Description = "供应商")]
        public string Supplier { get; set; }

        [ApiMember(Description = "产品颜色")]
        public string Color { get; set; }

        [ApiMember(Description = "成分")]
        public string Part { get; set; }

        [ApiMember(Description = "支付金额")]
        public decimal Money { get; set; }

        [ApiMember(Description = "检测单类型")]
        public string Type { get; set; }

        [ApiMember(Description = "原报告编号")]
        public string OriginalReportNo { get; set; }

        [ApiMember(Description = "原款号")]
        public string OriginalStyle { get; set; }

        [ApiMember(Description = "采购订单号")]
        public string OriginalPO { get; set; }

        public List<Detail> Detials { get; set; }

        private IDbTransaction _trans;
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new ConfirmResponse();
            var conn = KiviiContext.GetOpenedDbConnection<LaXiaOrder>();
            if (Token != "LX123456") {
                rtns.ResponseCode = 0;
                rtns.ResponseMsg = "客户未禁授权，请联系管理员授权访问！";
                return rtns;
            }
            if (conn.Exists<LaXiaOrder>(o => o.CheckNumbers == CheckNumbers)) {
                rtns.ResponseCode = 0;
                rtns.ResponseMsg = "检测单号已存在！上传失败";
                return rtns;
            } 
            _trans = conn.OpenTransaction();
            try
            {
                var OrderItem = new LaXiaOrder();
                var DetailItem = new LaXiaOrderDetail();
                OrderItem.Kvid = Guid.NewGuid();
                foreach (var detail in Detials)
                {
                    DetailItem.OrderKvid = OrderItem.Kvid;
                    DetailItem.Kvid = Guid.NewGuid();
                    DetailItem.name = detail.Name;
                    DetailItem.code = detail.Code;
                    DetailItem.part = detail.Part;
                    conn.Insert(DetailItem);
                }
                OrderItem.CheckNumbers = CheckNumbers;
                OrderItem.Name = Name;
                OrderItem.CBrand = CBrand;
                OrderItem.Brand = Brand;
                OrderItem.Style = Style;
                OrderItem.Standard = Standard;
                OrderItem.TestLevel = (5/TestLevel.ToInt()).ToString();
                OrderItem.TestInstitute = TestInstitute;
                OrderItem.Quarity = Quarity;
                OrderItem.Contains = Contains;
                OrderItem.PayType = PayType;
                OrderItem.TestType = TestType;
                OrderItem.Supplier = Supplier;
                OrderItem.Color = Color;
                OrderItem.Part = Part;
                OrderItem.Money = Money;
                OrderItem.Type = Type;
                OrderItem.OriginalReportNo = OriginalReportNo;
                OrderItem.OriginalStyle = OriginalStyle;
                OrderItem.OriginalPO = OriginalPO;
                conn.Insert(OrderItem);
                _trans.Commit();
                rtns.ResponseCode = 1;
                rtns.ResponseMsg = "正常接收";
            }
            catch (Exception ex)
            {
                if (_trans != null) _trans.Rollback();
                rtns.ResponseCode = 0;
                rtns.ResponseMsg = "接收失败,异常原因:"+ex;
            }
            return rtns;
        }
    }

    //lism业务人员确认收到拉夏寄过来的货物接口
    public class OrderStatusTo1 : RestfulExecution<LaXiaOrder>
    {
        public string CheckNumbers { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            LaXiaOrder order = null;
            var conn = KiviiContext.GetOpenedDbConnection<LaXiaOrder>();
            var queryOrder = conn.From<LaXiaOrder>().Where(o => o.CheckNumbers == CheckNumbers);
            order = conn.Select(queryOrder).FirstOrDefault();
            if (order == null) throw new Exception("未查询到该编号的信息!");
            else if (order.Status == 1) throw new Exception("已存在该编号的样品信息，请核实后操作！");
            else if (order.Status == 0)
            {
                order.Status = 1;
                order.AddOnlyProperties(o => o.Status);
                conn.UpdateOnly(order);
                order.RemoveAllOnlyProperties();
            }
            else throw new Exception("状态异常，无法操作，请联系管理员！");
            return order;
        }
    }

    public class TestPdf
    {
        public string FileName { get; set; }
        public string FilePath { get; set; }
    }

    //状态反馈至拉夏的接口 10：不接受 20：已接收 30：已修改 40：已完成
    public class TestResultResponse
    {
        public string Result { get; set; }
        public string Message { get; set; }
    }
    [Route("/plmservice/TestInstitute2scm")]
    public class TestResult:IReturn<TestResultResponse>
    {
        public string userName { get; set; }
        public string pwd { get; set; }
        public string testInstitute { get; set; }
        public string supplier { get; set; }
        public string CheckNumbers { get; set; }
        public string style { get; set; }
        public int status { get; set; }
        public string standard { get; set; }
        public string testLevel { get; set; }
        public string quarity { get; set; }
        public string contains { get; set; }
        public int result { get; set; }
        public List<LaXiaOrderDetail> detail { get; set; }
        public List<TestPdf> pdf { get; set; }
    }
    public class StatusBackToLaxia : RestfulExecution<LaXiaOrder>
    {
        public string CheckNumbers { get; set; }
        public int Status { get; set; }

        private IDbTransaction _trans;
        public override object OnExecution(IRequest req, IResponse res)
        {
            JsonServiceClient client = new JsonServiceClient("http://222.73.66.84:8089");

            var conn = KiviiContext.GetOpenedDbConnection<LaXiaOrder>();
            var query = conn.From<LaXiaOrder>().Where(o => o.CheckNumbers == CheckNumbers);
            var queryTest = conn.Select(query).FirstOrDefault();
            var rtns = new TestResultResponse();
            _trans = conn.OpenTransaction();
            try
            {

                if (queryTest == null)
                {
                    throw new Exception("操作异常，检测单号不存在！");
                }
                var testResult = new TestResult
                {
                    userName = "test",
                    pwd = "password",
                    testInstitute = queryTest.TestInstitute,
                    CheckNumbers = CheckNumbers,
                    supplier = queryTest.Supplier,
                    style = queryTest.Style,
                    status = Status
                };
                var connLims = KiviiContext.GetOpenedDbConnection<Contract>();
                var query2 = connLims.From<Contract>();
                //query2.Select(o => new { o.OrderID });
                var contractList = connLims.Select(query2);

                //只有在状态为已接收和已修改时，同步在lims系统中创建合同
                if (!contractList.Exists(o => o.OrderID == CheckNumbers)&&(Status==20||Status==30))
                {
                    var newContract = new Contract();
                    newContract.Kvid = Guid.NewGuid();
                    newContract.ReportId = "LX"+ DateTime.Now.Year.ToString()+ DateTime.Now.Hour.ToString()+ DateTime.Now.Minute.ToString()+ DateTime.Now.Second.ToString();
                    newContract.OrderID = CheckNumbers;
                    newContract.CustomerOrgName = queryTest.CBrand;
                    newContract.ProductBrand = queryTest.Brand;
                    newContract.ProductModel = queryTest.Style;
                    newContract.StandardOverView = queryTest.Standard;

                    var detailList = conn.Select<LaXiaOrderDetail>(o => o.OrderKvid == queryTest.Kvid);
                    var detailString="";
                    foreach(var detail in detailList)
                    {
                        detailString += detail.name+",";
                    }

                    var productSafetyGrade = queryTest.TestLevel == "1" ? "A等品" : queryTest.TestLevel == "2" ? "B等品" : queryTest.TestLevel == "3" ? "C等品" : "";
                    newContract.ProductSafetyGrade = queryTest.TestLevel;

                    var productGrade = queryTest.Quarity == "1" ? "合格品" : queryTest.Quarity == "2" ? "一等品" : queryTest.Quarity == "3" ? "优等品" : "";
                    newContract.ProductGrade = queryTest.Quarity;

                    newContract.Metadata["Eremark"] = $"检验依据：{queryTest.Standard+Environment.NewLine}检验项目：{detailString+Environment.NewLine}样品等级：{ productGrade+Environment.NewLine}安全类别：{ productSafetyGrade}";
                    connLims.Insert(newContract);


                    //同时更新该检测申请的 bizkvid为刚刚创建的合同
                    queryTest.BizKvid = newContract.Kvid;
                    queryTest.AddOnlyProperties(o => o.BizKvid);
                }
                if (Status == 30)//先更新好拉夏发送过来的数据之后才能调用30状态的已修改接口
                {
                    testResult.standard = queryTest.Standard;
                    testResult.testLevel = queryTest.TestLevel;
                    testResult.quarity = queryTest.Quarity;
                    testResult.contains = queryTest.Contains;
                }

                //状态：已完成
                if (Status == 40)
                {
                    var query3 = conn.From<LaXiaOrderDetail>().Where(o => o.OrderKvid == queryTest.Kvid);
                    var queryTestDetailList = conn.Select(query3).ToList();

                    var currentContract = connLims.Select<Contract>(o => o.Kvid == queryTest.BizKvid).FirstOrDefault();

                    testResult.standard = currentContract.StandardOverView;
                    testResult.testLevel = currentContract.ProductSafetyGrade;
                    testResult.quarity = currentContract.ProductGrade;
                    testResult.contains = currentContract.ProductName;

                    //查询所有检测项目中是否有不合格的，来返回result给拉夏接口，1：通过，2：不通过
                    var query4 = conn.From<Lims.Entities.Task>().Where(o => o.ContractKvid == queryTest.BizKvid);
                    var taskList = connLims.Select(query4);
                    if (taskList.Exists(o => o.IsQualified == "不合格"))
                    {
                        testResult.result = 2;
                    }
                    else
                    {
                        testResult.result = 1;
                    }

                    var testDetailList = new List<LaXiaOrderDetail>();
                    //查询已完成的检测，去查他的项目，然后到项目对应表中去查对应的项目名称和编号
                    //var groups = taskList.
                    //var query5 = conn.From<TestItem>();
                    var testItemList = conn.Select<TestItem>();
                    foreach(var task in taskList)
                    {
                        var laxiaOrderDetail = new LaXiaOrderDetail();
                            laxiaOrderDetail.type = "理化";
                            laxiaOrderDetail.practialWay = task.TestMethod;
                            laxiaOrderDetail.practialResult = task.MeasuredValue;
                            laxiaOrderDetail.practialAdvice = task.IsQualified;
                            laxiaOrderDetail.part = queryTest.Part;
                            laxiaOrderDetail.color = queryTest.Color;
                            laxiaOrderDetail.requirement = task.ReferenceValue;
                            laxiaOrderDetail.cost = Math.Round(task.RealFee,2,MidpointRounding.AwayFromZero).ToString();
                            laxiaOrderDetail.unit = task.Unit;
                            //处理Required为1：要求检测和2：新增检测
                            if (queryTestDetailList.Exists(o => o.name == task.Title))
                            {

                                laxiaOrderDetail.required = "1";
                            }
                            else
                            {
                                laxiaOrderDetail.required = "2";
                            }
                        //处理返回的项目对应表中的名称和对应的编码
                        //处理大项
                        if (task.ParentKvid == Guid.Empty)
                        {
                            laxiaOrderDetail.name = task.Title;
                            if (testItemList.FirstOrDefault(o => o.Name == task.Title)==null)
                            {
                                laxiaOrderDetail.code = "000000";
                            }
                            else
                            {
                                laxiaOrderDetail.code=testItemList.FirstOrDefault(o => o.Name == task.Title).Code;
                            }
                        }
                        //处理小项
                        else
                        {
                            laxiaOrderDetail.name = task.TitleEx;
                            testItemList = testItemList.Where(o => o.ParentName == task.Title).ToList();
                            if(testItemList.FirstOrDefault(o => o.Name == task.TitleEx) == null)
                            {
                                laxiaOrderDetail.code = "000000";
                            }
                            else
                            {
                                laxiaOrderDetail.code = testItemList.FirstOrDefault(o => o.Name == task.TitleEx).Code;
                            }
                        }
                        testDetailList.Add(laxiaOrderDetail);
                    }
                    //处理Required为3:删除检测
                    foreach(var testDetail in queryTestDetailList)
                    {
                        if (!taskList.Exists(o => o.Title == testDetail.name)||!taskList.Exists(o=>o.TitleEx==testDetail.name))
                        {
                            var laxiaOrderDetail = new LaXiaOrderDetail();
                            laxiaOrderDetail = testDetail;
                            laxiaOrderDetail.required = "3";
                            testDetailList.Add(laxiaOrderDetail);
                        }
                    }
                    testResult.detail = testDetailList;
                    //处理返回pdf,先要查到该合同
                    var testPdfList = new List<TestPdf>();
                    var testPdf = new TestPdf();
                    testPdf.FileName = "检测结果pdf";
                    testPdf.FilePath = "http://apps.jst-gov.com/Reports/" + currentContract.Kvid + "/" + currentContract.ReportId + ".pdf";
                    testPdfList.Add(testPdf);
                    testResult.pdf = testPdfList;
                }
                //数据返回给拉夏后，同步更新原拉夏发送的数据状态
                if(Status == 10 || Status == 20 || Status == 30 || Status == 40)
                {
                    queryTest.Status = Status;
                    queryTest.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(queryTest);
                }
                else
                {
                    throw new Exception("非法参数！");
                }

                rtns = client.Post(testResult);
                _trans.Commit();
                _trans.Dispose();
            }
            catch(Exception ex)
            {
                _trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }
}
